/*********************************************************************
  CAPCOMx_2.c          												 
  CAPCOMx-Unit (x = 1 or 2) in the compare mode (CCMODx = 1xx) for 
  makes of signals with Capture/Compare registers CC0 ... CC5,CC13
  in several compare modes. CC0 works in compare mode 0, CC1 and CC2
  in mode 1, CC3 and CC4 in mode 3 and CC5 with CC13 in Doble Register
  Compare Mode (See file CAPCOMx_2.gif).  
  Signal outputs on pins P2.0 ... P2.5 
   
                            	       PHYTEC Messtechnik GmbH, 2000
 *********************************************************************/

/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* Register defin for C167              */
#include "LED.h"
 
_sfrbit DCC0IO _atbit(DP2,0);
_sfrbit DCC1IO _atbit(DP2,1);
_sfrbit DCC2IO _atbit(DP2,2);
_sfrbit DCC3IO _atbit(DP2,3);
_sfrbit DCC4IO _atbit(DP2,4);
_sfrbit DCC5IO _atbit(DP2,5);

#define CC0out DCC0IO = 1
#define CC1out DCC1IO = 1
#define CC2out DCC2IO = 1
#define CC3out DCC3IO = 1
#define CC4out DCC4IO = 1
#define CC5out DCC5IO = 1

/* ----- Compare Interrupt Program for CC0 ------------------------- */
_interrupt (0x10) void Compare (void) 
{
 LED = !LED;
}

/* ------ main Program: -------------------------------------------- */
void main (void)
{

 LED_on;                       /* enable LED on PHYTEC board         */
 CC0out; CC1out; CC2out; 
 CC3out; CC4out; CC5out;       /* pin direction: output              */
  
 /* ------------------------------------------------------------ */
 /* x = 0, 1, 7, 8,                Fcpu = 20 MHz                 */
 /* ------------------------------------------------------------ */
 /* TxI | TxI |         Input          | Resolution | Overflow   */
 /* d/h | bin |         Clock          |            |  Period    */
 /* ------------------------------------------------------------ */
 /*  0  | 000 | Fcpu/8    =   2.50 MHz |    400 ns  |  26.21 ms  */
 /*  1  | 001 | Fcpu/16   =   1.25 MHz |    800 ns  |  52.43 ms  */
 /*  2  | 010 | Fcpu/32   = 625.00 kHz |   1600 ns  | 104.86 ms  */
 /*  3  | 011 | Fcpu/64   = 312.50 kHz |   3200 ns  | 209.71 ms  */
 /*  4  | 100 | Fcpu/128  = 156.25 kHz |   6400 ns  | 419.43 ms  */
 /*  5  | 101 | Fcpu/256  =  78.125kHz |  12800 ns  | 838.86 ms  */
 /*  6  | 110 | Fcpu/512  =  39.06 kHz |  25600 ns  |   1.677 s  */
 /*  7  | 111 | Fcpu/1024 =  19.53 kHz |  51200 ns  |   3.355 s  */
 /* ------------------------------------------------------------ */ 
 /*    Overflow Period  = 1/[Fcpu/(8*(2^TxI)*(2^16)]             */
 /* ------------------------------------------------------------ */

  /* init Timer 0                                                    */
  T01CON = T01CON & 0xFF00 | 0x0001;/*T0 as timer with Fcpu/16 input:*/
  T0REL  = 59286;            /* reload value fot T0: (65535- 59286)*800
				= 5 ms => 200 Hz                     */
  /* init Timer 1                                                    */
  T01CON = T01CON & 0x00FF | 0x0100;/*T1 as timer with Fcpu/16 input:*/
  T1REL  = -6250;	     /* reload value fot T1: 6250*800 = 5 ms
  				=> 200 Hz                            */

  /* CC0 in Compare Mode 0: interrupt only                           */
  CCM0 = CCM0 & 0xFFF0 | 0x0004;/* init CC0 register for compare works 
				with T0 (software timer)             */
  CC0  = 1562+59286;         /* compare value                        */
  CC0IC = 0x0054;            /* enable interrupt for CC0:
				ILVL = 5; GLVL = 0; CC0IE = 1.       */

  /* CC1 and CC2 in Compare Mode 1: interrupt and pin toggeln        */
  CCM0 = CCM0 & 0xFF0F | 0x0050;/* init CC1 register for compare mode 1
				works with T0                        */
  CC1 = 1562+59286;          /* compare value: Fout = 100 Hz, 
				HIGH pegel = LOW  pegel = 2.5 ms     */
  
  CCM0 = CCM0 & 0xF0FF | 0x0500;/* init CC2 register for compare mode 1
				works with T0                        */
  CC2 = 4687+59286;          /* compare value: Fout = 100 Hz, 
				HIGH pegel = LOW  pegel = 2.5 ms     */

  /* CC3 and CC4 in Compare Mode 3: interrpt and pin toggeln (PWM)   */
  _bfld(CCM0,0xF000,0xF000); /* init CC3 register for compare mode 3
				works with T1                        */
  CC3 = 1562+59286;          /* compare value: Fout = 200 Hz
                                HIGH pegel = 3.75 ms
                                LOW pegel  = 1.25 ms                 */
  _bfld(CCM1,0x000F,0x000F); /* init CC4 register for compare mode 3
				works with T1                        */
  CC4 = 4687+59286;          /* compare value: Fout = 200 Hz
				HIGH pegel = 1.25 ms
                                LOW pegel  = 3.75 ms                 */

  /* CC5 and CC13 works in Doble Register Compare Mode               */
  _bfld(CCM1,0x00F0,0x0050); /* init CC5 register for compare mode 1
                                works with T0                        */ 
  CC5 = 1562+59286;          /* compare value: Fout = 200 Hz         */
  _bfld(CCM3,0x00F0,0x0040); /* init CC13 register for compare mode 0
				works with T0			     */
  CC13 = 4687+59286;	     /* compare value:			     */
				
  IEN   = 1;		     /* enable global interrupt              */

  T01CON |= 0x4040;	     /* start Timer T0 and T1                */

  while (1)                  /* endless loop	                     */
   {
     /* here your program    		                             */
   } 
}

