/*********************************************************************
  GPT1_3.c          												 
   Timer T3 from General Purpose Timer Block 1 as Timer in gated mode
   for measuring of high-impuls width with input clock Fcpu/8 = 400 ns.
   Timer T4 initialize as a counter with imput from T3OTL and with 
   timer T3 work that as a 32-bit timer for measuring of long impulses
   (width < 4 sek.).
   
                            		 PHYTEC Messtechnik GmbH, 2000
 *********************************************************************/

/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* Register defin for C167              */
#include <STDIO.H>           /* Standard I/O functions               */
#include "serio.h"           /* init serial interface                */
#include "PWM.h"             /* init PWM - Unit                      */
 
_sfrbit DT3IN  _atbit(DP3,6);
unsigned char T3I;
unsigned long Width;

/* ------ main Program: -------------------------------------------- */
void main (void)
{

  init_serio(57600);         /* init serial interface with 57600baud */
  PWM();		     /* start PWM-Unit for test signal       */

  DT3IN  = 0;                /* pin direction: input                 */
  
             /* ---------------------------------------------------- */
             /*                     Fcpu = 20 MHz                    */
             /* ---------------------------------------------------- */
             /* T3I |   Input   | Overflow  |  Output   |   Output   */
	     /* bin |   Clock   |  Period   | Frequency |   Period   */
             /* ---------------------------------------------------- */
    T3I = 0; /* 000 | Fcpu/8    |  26.21 ms | 19.073 Hz |  52.428 ms */
//  T3I = 1; /* 001 | Fcpu/16   |  52.43 ms |  9.536 Hz | 104.857 ms */
//  T3I = 2; /* 010 | Fcpu/32   | 104.86 ms |  4.768 Hz | 209.715 ms */
//  T3I = 3; /* 011 | Fcpu/64   | 209.71 ms |  2.284 Hz | 419.430 ms */
//  T3I = 4; /* 100 | Fcpu/128  | 419.43 ms |  1.192 Hz | 838.861 ms */
//  T3I = 5; /* 101 | Fcpu/256  | 838.86 ms |  0.596 Hz |   1.677 s  */
//  T3I = 6; /* 110 | Fcpu/512  |   1.677 s |  0.298 Hz |   3.355 s  */
//  T3I = 7; /* 111 | Fcpu/1024 |   3.355 s |  0.149 Hz |   6.710 s  */
             /* ---------------------------------------------------- */ 
             /*    Overflow Period  = 1/[Fcpu/(8*(2^T3I)*(2^16)]     */
	     /*    Output Frequency = Fcpu/[8*(2^T3I)*(2^16)*2]      */
	     /*    Output Period    = 1/{Fcpu/[8*(2^T3I)*(2^16)*2]}  */
	     /* ---------------------------------------------------- */

  T3CON = 0x0018 | T3I;	     /* T3 as timer with Fcpu/8 = 400 ns input   
			     	in gated mode (High active)          */

  T4CON = 0x000F;            /* T4 as counter with input clock from
                                T3OTL. (T3 + T4 as 32-bit Timer )    */
   /* or  */
/* T4CON = 0x000D            /* T4 as counter with input clock from
        or 0x000E               T3OTL. (T3 + T4 as 33-bit Timer )    */

  T3 = T4 = 0;		     /* start value for Timer T3 and T4      */

  printf("\n");

   while (1)                 /* endless loop	                     */
   {
     /* here your program    	                                     */
     if(T3IN)			 
     {
       while(T3IN);          /* wait as long as 'T3IN' is High	     */
       T3R = T4R = 0;        /* stop Timer T3 and T4                 */
       Width =((long)(65535)* T4 +(long)(T3))*400;
       printf("\r Width of High Impulse = %lu ns,( T3 = %x hex., T4 = %x hex.) ", Width,T3,T4);
       T3 = T4 = 0;	     /* clear Timer T3 and T4                */
       while(T3IN);          /* wait as long as 'T3IN' is High	     */
       T3R = T4R = 1;        /* stert Timer T3 and T4                */
     }
     
	 /* here your program    		                     */
   } 
}

