/*********************************************************************
  PWM.c          												 
                                                                 
  The PWM-Unit generate  PWM-Signals on PWM-Channel 0 - 3 Center 
  Aligned PWM with constant frequecy (period) and puls width from 1/2 
  of period.
  All PWM-Timer 1 work with CPU-Clock (20 MHz)                     
  The period register PP0 contain the value 128:                     
  Frequency a PWM-Signal 0 = 20 MHz / 2 * (128-1) = 78,740 kHz
  The period register PP1 =  1000, Fpwm1 = 10.010 kHz,
                      PP2 = 10000, Fpwm2 = 1.000  kHz,
 		      PP3 = 50000, Fpwm3 = 200.004 Hz.
  Output at Port pin P7.0/POUT0 - P7.3/POUT3 High-aktive (pin B40,
  B42, B44, B46) is enabled.
 
			    	     PHYTEC Messtechnik GmbH, 2000
 *********************************************************************/

/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* Register defin for C167              */
								
_sfrbit DP70  _atbit(DP7,0); /* Bitname DP70 for DP7.0 define        */
_sfrbit DP71  _atbit(DP7,1); /* Bitname DP71 for DP7.1 define        */
_sfrbit DP72  _atbit(DP7,2); /* Bitname DP72 for DP7.2 define        */
_sfrbit DP73  _atbit(DP7,3); /* Bitname DP73 for DP7.3 define        */

/* ------ main Program: -------------------------------------------  */
void PWM (void)
{  
   DP70 = DP71 = DP72 = DP73 = 1; /* Define PWM-Output as Push-/Pull-
                                Output, Latch-contents after Reset 0 */
   PP0   = 128;	             /* Load period register                 */
   PW0   = 64;               /* Load puls width register             */
   PP1   = 1000;	     /* Load period register                 */
   PW1   = 500;              /* Load puls width register             */
   PP2   = 10000;	     /* Load period register                 */
   PW2   = 5000;             /* Load puls width register             */
   PP3   = 50000;	     /* Load period register                 */
   PW3   = 25000;            /* Load puls width register             */

   PWMCON1 = 0x00FF;         /* init PWM: center aligned PWM, output 
   			     	siglals enable                       */
   PWMCON0 = 0x000F;	     /* start PWM Timers with clock = Fcpu   */
}
/* -- End of program ----------------------------------------------- */
