/******************************************************************************
   serio.c                                                
                                                                        
  Implementation of _ioread() and _iowrite() using the low level 
  I/O functions getch(), putch(), kbhit() and init_serio(BaudRate) working
  with serial channel 0 or channel 1 of the SAB 80C166. Default is serial 
  channel 0 used for I/O. If serial channel 1 is wanted for I/O compile	  
  "serio.c" with the -DSER_PORT_1 option.		  
                          
                                               PHYTEC Messtechnik GmbH, 2000                                                                        
*******************************************************************************/
#include <stdio.h>

#define REG166_NOADC		/* exclude ADC SFRs			      */
#define REG166_NOCAPCOM		/* exclude CAPCOM SFRs 			      */
#define REG166_NOCPU		/* exclude CPU SFRs 			      */
#define REG166_NOPEC		/* exclude PEC SFRs 			      */
#define REG166_NOTIMER		/* exclude timer SFRs 			      */
#include <reg166.h>

#include "SERIO.H"

/* S0/1 port: 8 bit, asynchronuous operation, one stopbit, receive enable, 
              no parity/frame/overrun check baud rate generator enable        */

void init_serio( unsigned long BaudRate )
{
/* init serial0 port: Asynchronous Mode 8-bit Data, one Stop bit, no Parity   */  
  P3  |= 0x0400;                       /* set port 3.10 output latch (TXD)    */
  DP3 |= 0x0400;                       /* configure port 3.10 for output      */
                                       /* operation. ( TXD output)            */
  DP3 &= 0xF7FF;                       /* configure port 3.11 for input       */
                                       /* operation. ( RXD input)             */
  SXTIC  = 0x80;                       /* set transmit interrupt flag         */
  SXRIC  = 0x00;                       /* delete receive interrupt flag       */
  SXCON  = 0x0011;                     /* set serial mode                     */

  switch (BaudRate)
  {
   case     50:  SXBG   = 0x1FFF;      /* set baudrate to   50 baud           */
                 SXCON |= 0x2000;      // Baud Rate Generator frequency = fCPU/3
			     break;
   case     75:  SXBG   = 0x1FFF; break;   /* set baudrate to     75 baud     */
   case    110:  SXBG   = 0x1630; break;   /* set baudrate to    110 baud     */
   case    300:  SXBG   = 0x0822; break;   /* set baudrate to    300 baud     */
   case    600:  SXBG   = 0x0410; break;   /* set baudrate to    600 baud     */
   case   1200:  SXBG   = 0x0207; break;   /* set baudrate to   1200 baud     */
   case   2400:  SXBG   = 0x0103; break;   /* set baudrate to   2400 baud     */
   case   4800:  SXBG   = 0x0081; break;   /* set baudrate to   4800 baud     */
   case   9600:  SXBG   = 0x0040; break;   /* set baudrate to   9600 baud     */
   case  19200:  SXBG   = 0x001F; break;   /* set baudrate to  19200 baud     */
   case  38400:  SXBG   = 0x000F; break;   /* set baudrate to  38400 baud     */
   case  57600:  SXBG   = 0x000A; break;   /* set baudrate to  57600 baud     */
   case 625000:  SXBG   = 0x0000; break;   /* set baudrate to 625000 baud     */
   }

  SXR = 1;                             /* start ASC0			      */

/*  end of init serial0 port:         					      */
}

/* Read character from serial channel                                         */
int getch( void )
{
	int c = EOF;

	if ( SXEIR )
	{
		SXPE = 0;
		SXFE = 0;
		SXEIR = 0;
		SXRIR = 0;
	}
	else if ( SXRIR )
	{
		c = SXRBUF & 0x7F;
		SXRIR = 0;
	}
	return ( c );
}

/* Return 1 if character available, otherwise 0                               */
int kbhit( void )
{
	if ( SXRIR )
		return ( 1 );
	return ( 0 );
}

/* Write character to serial channel                                          */
int putch( int c )
{
	while ( ! SXTIR ) 
		;
	SXTIR  = 0;
	SXTBUF = c;
	return ( c );
}

int _ioread( FILE * fin )
{
        /* At this point, it is known that 'fin' is a file opened for         */
        /* reading, and is not in error yet.                                  */
        /* This routine should read the input for the given filehandle        */
        /* (in the standard version only 'stdin') from the required           */
        /* hardware.                                                          */
        /* Whenever an error occurs, 'EOF' should be returned                 */
        /* and fin->_flag should be set to the proper error value             */
        /* i.e.         _IOEOF on end of FILE                                 */
        /*              _IOERR on any other error                             */
        /* Note that when a read is done on 'stdin', this routine             */
        /* should do the scanning, and whenever neccessary, do the            */
        /* echoing to stdout                                                  */

	int c = EOF;

        if ( fin == stdin )
        {
        	/* Assume terminal sending either CR or CRLF. Allow only LF as
        	   valid delimiter, so translate CR into LF and ignore received
        	   LF's                                                       */
                do	/* blocking read from serial channel 0                */
		{
			c = getch();		/* without echo               */
			if ( c == 0x0a )	/* LF                         */
				c = EOF;	/* ignore it                  */
			else if ( c == 0x0d )	/* CR                         */
				c = 0x0a;	/* turn into LF               */
		}
		while ( c < 0 );

		/* when reading 'stdin', echo to 'stdout' performing 
		   LF ==> CR-LF translation                                   */
                _iowrite( c, stdout );
        }
	return( c );	/* return read character on succes                    */
}


int _iowrite( int c, FILE * fout )
{
        /* At this point, it is known that 'fout' is a file opened for        */
        /* writing, and is not in error yet.                                  */
        /* This routine should write the output for the given filehandle      */
        /* (in the standard version only 'stdout' or 'stderr')                */
        /* to the required hardware.                                          */
        /* Whenever an error occurs, 'EOF' should be returned                 */
        /* and fout->_flag should be set to the proper error value            */
        /* i.e.         _IOEOF on end of FILE                                 */
        /*              _IOERR on any other error                             */

	if ( fout == stdout  ||  fout == stderr )
	{
		if ( c == '\n' )
			putch( '\r' );
		putch( c );
	}

        return( c );    /* return written character on success                */
}
