/*****************************************************************************
   ASC0_4.c
                                                                    
   The ASC0 receive to a buffer and transmit from a buffer 20 Byte each 
   (8 Bit, no Parity) by help from PEC transfer.
   The asynchronous serial interface work in mode 1 Baudrate: 9600 Baud.
   S0RIR release 20 ISR and write over PEC all receive chars to buffer s0_buffer.
   After thet stop the receive, enable a transmit interrupt for new
   transmit and initialze PEC Cahnnel 7 for new transfer.        
   S0TBIR release 20 ISR, read over PEC all chars from buffer s0_buffer and 
   transmit it over serial interface. After thet stop trensmit, initialize 
   interrupt control register SORIC for new receive PEC Channel 6 for new 
   transfer.      
   
   S0TBIC: ILVL = 15, GLVL = 2, S0TBIE = 1 => PEC Channel 6                            
   S0RIC:  ILVL = 15, GLVL = 3, S0RIE  = 1 => PEC Channel 7                             
   The three Error Interrupts is not enable.
    
						 PHYTEC Messtechnik GmbH, 2000
 ******************************************************************************/
/* ----- Include Dateien: --------------------------------------------------- */
#pragma align sb=c.
#include <REG167CR.H>          	    /* Register Definition for C167           */

/* ----- C-globale Byte-/Wort-Objekte: -------------------------------------- */
#define BUFFER_VALUE 20

#pragma class fb=firstsegment
int _far s0_buffer[BUFFER_VALUE];   /* unsigned 16 Bit C-Array                */

/* ----- S0Receive-Interrupt program: --------------------------------------- */
_interrupt (0x2b) void s0rint (void) 
{  									                                          
   /* Here your program, example		  	             	      */
   S0RIC = 0;			    /* Disable receive interrupt	      */ 
   PECC7 = 0x300 + BUFFER_VALUE;    /* Increment DSTP7, Byte-Transfer         */
   SRCP7 = (int)&S0RBUF;    	    /* Source pointer = 16 Bit Address        */
   DSTP7 = (int)s0_buffer;  	    /* Destination pointer=16 Bit Address     */

   S0TBIC = 0x78 + 6;               /* Init transmit buffer interrupt for new */
				    /* transmit 			      */	 
   S0TBIR = 1;			    /* Start first transmit                   */
}

/* ----- S0Transmit-Interrupt program: -------------------------------------- */
_interrupt (0x47) void s0tint (void) 
{  
   /* Here your program, example					      */
   S0TBIC = 0;                	    /* Disable transmit interrupt             */
   PECC6 = 0x500 + BUFFER_VALUE;    /* Increment SRCP6, Byte-Transfer,        */
   SRCP6 = (int)s0_buffer;          /* Source pointer = 16 Bit Address        */
   DSTP6 = (int)&S0TBUF;            /* Destination pointer=16 Bit Address     */
  
   S0RIC = 0x78 + 7;		    /* Init receive interrupt for nuw receive */
}

/* ----- main Programm: ----------------------------------------------------- */
void main ( )
{  
   P3  |= 0x0400;             	    /* Set port 3.10 output latch (TXD)       */
   DP3 |= 0x0400;             	    /* Configure port 3.10 for output         */
                              	    /* Operation. ( TXD output)               */
   DP3 &= 0xF7FF;             	    /* Configure port 3.11 for input          */
                              	    /* Operation. ( RXD input)                */

   S0TBIC = 0x78 + 6;               /* Init S0TBuffer-Interrupt for PEC 6     */
   PECC6 = 0x500 + BUFFER_VALUE;    /* Increment SRCP6, Byte-Transfer,        */
   SRCP6 = (int)s0_buffer;          /* Source pointer = 16 Bit Address        */
   DSTP6 = (int)&S0TBUF;            /* Destination pointer = 16 Bit Address   */

   S0RIC  = 0x78 + 7;          	    /* Init S0Receive-Interrupt for PEC 7     */
   PECC7 = 0x300 + BUFFER_VALUE;    /* Increment DSTP7, Byte-Transfer         */
   SRCP7 = (int)&S0RBUF;            /* Source pointer = 16 Bit Address        */
   DSTP7 = (int)s0_buffer;          /* Destination pointer = 16 Bit Address   */
   
   S0BG  = 0x40;                    /* Load baudraten generator for 9600 baud */
   S0CON = 0x8011;                  /* Init serial interface ASC0             */

   IEN   = 1;                       /* Eanble globale interrupt               */   
   
   while(1)                   	    /* Endless loop                           */
   {
      _idle();                      /* CPU in the Idle-Mode                   */
   } 
}/* ----- End: -------------------------------------------------------------- */
