//****************************************************************************
// @Module        CAN1
// @Filename      CAN1.C
// @Project       7CAN_2
//----------------------------------------------------------------------------
// @Controller    Siemens C167CR-L
//
// @Compiler      Tasking C166 V5.0
//
// @Codegenerator DAvE 1-0-0
//
// @Description   This file contains all functions that use the CAN1 peripheral
//                
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:11 PM
//
//****************************************************************************

//****************************************************************************
// @Project Includes
//****************************************************************************

#include "MAIN.H"


// USER CODE BEGIN (CAN1_General,1)

// USER CODE END


//***************************************************************************
// @Global Variables
//***************************************************************************

// USER CODE BEGIN (CAN1_General,2)

// USER CODE END


//****************************************************************************
// @Function      void CAN1_vInit(void)
//
//----------------------------------------------------------------------------
// @Description   This function initializes the CAN1 component. It effects all
//                necessary configurations of the SFR, depending on the selected
//                operating mode. The configuration determines whether the CAN1
//                interrupts are to be released, and the priority of the 
//                released interrupt.
//
//----------------------------------------------------------------------------
// @Returnvalue   none
//
//----------------------------------------------------------------------------
// @Parameters    none
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:11 PM
//
//****************************************************************************

void CAN1_vInit(void)
{

  //  ------------ CAN1 Control/Status Register --------------
  //  start the initialization of the CAN Modul
  C1CSR1  = 0x0041;  // set INIT and CCE

  ///  ------------ Bit Timing Register ---------------------
  /// baudrate =  100.000 KBaud
  /// there are 5 time quanta before sample point
  /// there are 4 time quanta after sample point
  /// the (re)synchronization jump width is 2 time quanta
  C1BTR1  = 0x3449;  // set Bit Timing Register

  C1GMS1  = 0x0000;  // set Global Mask Short Register

  C1UGML1 = 0xFFFF;  // set Upper Global Mask Long Register

  C1LGML1 = 0xF8FF;  // set Lower Global Mask Long Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 1 ---------------------------
  //  ------------------------------------------------------------------------
  /// Message object 1 is valid
  CAN1_OBJ[0].MCR  = 0x5695;    // set Message Control Register

  /// message direction is transmit
  /// standard 11-bit identifier
  /// 4 valid data bytes
  CAN1_OBJ[0].MCFG = 0x48;      // set Message Configuration Register

  CAN1_OBJ[0].UAR  = 0x2002;    // set Upper Arbitration Register
  CAN1_OBJ[0].LAR  = 0x0000;    // set Lower Arbitration Register

  CAN1_OBJ[0].Data[0] = 0x0A;   // set data byte 0
  CAN1_OBJ[0].Data[1] = 0x0B;   // set data byte 1
  CAN1_OBJ[0].Data[2] = 0x0C;   // set data byte 2
  CAN1_OBJ[0].Data[3] = 0x0D;   // set data byte 3
 
 
  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 2 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 3 is not valid
  CAN1_OBJ[2].MCR  = 0x5555;    // set Message Control Register



  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 3 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 3 is not valid
  CAN1_OBJ[2].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 4 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 4 is not valid
  CAN1_OBJ[3].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 5 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 5 is not valid
  CAN1_OBJ[4].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 6 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 6 is not valid
  CAN1_OBJ[5].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 7 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 7 is not valid
  CAN1_OBJ[6].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 8 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 8 is not valid
  CAN1_OBJ[7].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 9 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 9 is not valid
  CAN1_OBJ[8].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 10 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 10 is not valid
  CAN1_OBJ[9].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 11 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 11 is not valid
  CAN1_OBJ[10].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 12 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 12 is not valid
  CAN1_OBJ[11].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 13 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 13 is not valid
  CAN1_OBJ[12].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 14 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 14 is not valid
  CAN1_OBJ[13].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 15 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 15 is not valid
  CAN1_OBJ[14].MCR  = 0x5555;    // set Message Control Register

  /// ------------ CAN Control/Status Register --------------
  //  reset CCE and INIT
  C1CSR1 = 0x0000;


  // USER CODE BEGIN (CAN1_Init,1)

  // USER CODE END

}

//****************************************************************************
// @Function      void CAN1_vTransmit(ubyte ObjNr)
//
//----------------------------------------------------------------------------
// @Description   This function triggers the CAN1 controller to send the 
//                selected message.
//                If the selected message object is a TRANSMIT OBJECT then 
//                this function triggers the sending of a data frame.
//                If however the selected message object is a RECEIVE OBJECT
//                this function triggers the sending of a remote frame.
//
//----------------------------------------------------------------------------
// @Returnvalue   none
//
//----------------------------------------------------------------------------
// @Parameters    Number of the message object to be sent (1-14)
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:11 PM
//
//****************************************************************************

void CAN1_vTransmit(ubyte ObjNr)
{
  CAN1_OBJ[ObjNr-1].MCR = 0xe7ff;  // set TXRQ,reset CPUUPD
}




// USER CODE BEGIN (CAN1_General,3)

// USER CODE END


