//****************************************************************************
// @Module        CAN2
// @Filename      CAN2.C
// @Project       7CAN_2
//----------------------------------------------------------------------------
// @Controller    Siemens C167CR-L
//
// @Compiler      Tasking C166 V5.0
//
// @Codegenerator DAvE 1-0-0
//
// @Description   This file contains all functions that use the CAN2 peripheral
//                
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:11 PM
//
//****************************************************************************

//****************************************************************************
// @Project Includes
//****************************************************************************

#include "MAIN.H"


// USER CODE BEGIN (CAN2_General,1)

// USER CODE END


//***************************************************************************
// @Global Variables
//***************************************************************************

// USER CODE BEGIN (CAN2_General,2)
// USER CODE END


//****************************************************************************
// @Function      void CAN2_vInit(void)
//
//----------------------------------------------------------------------------
// @Description   This function initializes the CAN2 component. It effects all
//                necessary configurations of the SFR, depending on the selected
//                operating mode. The configuration determines whether the CAN2
//                interrupts are to be released, and the priority of the 
//                released interrupt.
//
//----------------------------------------------------------------------------
// @Returnvalue   none
//
//----------------------------------------------------------------------------
// @Parameters    none
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:11 PM
//
//****************************************************************************

void CAN2_vInit(void)
{

  //  ------------ CAN Control/Status Register --------------
  //  start the initialization of the CAN2 Module
  C1CSR2  = 0x0041;  // set INIT and CCE

  ///  ------------ Bit Timing Register ---------------------
  /// baudrate =  100.000 KBaud
  /// there are 5 time quanta before sample point
  /// there are 4 time quanta after sample point
  /// the (re)synchronization jump width is 2 time quanta
  C1BTR2  = 0x3449;  // set Bit Timing Register

  C1GMS2  = 0x0000;  // set Global Mask Short Register

  C1UGML2 = 0xFFFF;  // set Upper Global Mask Long Register

  C1LGML2 = 0xF8FF;  // set Lower Global Mask Long Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 1 ---------------------------
  //  ------------------------------------------------------------------------
  /// Message object 1 is valid
  /// enable receive interrupt
  CAN2_OBJ[0].MCR  = 0x5599;    // set Message Control Register

  /// message direction is receive
  /// standard 11-bit identifier
  CAN2_OBJ[0].MCFG = 0x00;      // set Message Configuration Register

  CAN2_OBJ[0].UAR  = 0x2002;    // set Upper Arbitration Register
  CAN2_OBJ[0].LAR  = 0x0000;    // set Lower Arbitration Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 2 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 2 is not valid
  CAN2_OBJ[1].MCR  = 0x5555;    // set Message Control Register




  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 3 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 3 is not valid
  CAN2_OBJ[2].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 4 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 4 is not valid
  CAN2_OBJ[3].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 5 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 5 is not valid
  CAN2_OBJ[4].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 6 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 6 is not valid
  CAN2_OBJ[5].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 7 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 7 is not valid
  CAN2_OBJ[6].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 8 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 8 is not valid
  CAN2_OBJ[7].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 9 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 9 is not valid
  CAN2_OBJ[8].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 10 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 10 is not valid
  CAN2_OBJ[9].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 11 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 11 is not valid
  CAN2_OBJ[10].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 12 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 12 is not valid
  CAN2_OBJ[11].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 13 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 13 is not valid
  CAN2_OBJ[12].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 14 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 14 is not valid
  CAN2_OBJ[13].MCR  = 0x5555;    // set Message Control Register


  //  ------------------------------------------------------------------------
  /// ----------------- Configure Message Object 15 ---------------------------
  //  ------------------------------------------------------------------------
  /// message object 15 is not valid
  CAN2_OBJ[14].MCR  = 0x5555;    // set Message Control Register


  ///  enable CAN interrupt
  ///  CAN interrupt priority level(ILVL) = 13
  ///  CAN interrupt group level (GLVL) = 3
  XP1IC = 0x0077;

  /// ------------ CAN Control/Status Register --------------
  //  reset CCE and INIT
  /// enable interrupt generation from CAN Modul
  C1CSR2 = 0x0002;


  // USER CODE BEGIN (CAN_Init,1)

  // USER CODE END

}


//****************************************************************************
// @Function      interrupt (XP0INT) void CAN_viIsr(void)
//
//----------------------------------------------------------------------------
// @Description   This is the interrupt service routine for the CAN2 controller.
//                Depending on the configuration it is executed if:
//                - the busoff or the error warning status is reached 
//                  (EIE is set)
//                - a message has been sent or received successfully or a bus 
//                  error occurred (SIE is set)
//                - the bit INTPND (interrupt pending) in one of the message
//                  object control-registers is set (at Tx or Rx)
//                
//                Note:
//                It is important that the user adds to this function with
//                Application Specific Code. User Code sections are available
//                in the function for this purpose.
//
//----------------------------------------------------------------------------
// @Returnvalue   none
//
//----------------------------------------------------------------------------
// @Parameters    none
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:11 PM
//
//****************************************************************************

interrupt (XP1INT) void CAN2_viIsr(void)
{
  uword uwIntID;
  while (uwIntID = C1IR2 & 0x00ff)
  {
    switch (uwIntID & 0x00ff)
    {
      case 3: // Message Object 1 Interrupt

              if ((CAN2_OBJ[0].MCR & 0x0300) == 0x0200)    // if NEWDAT set
              {

                if ((CAN2_OBJ[0].MCR & 0x0c00) == 0x0800)  // if MSGLST set
                {
                  // Indicates that the CAN controller has stored a new 
                  // message into this object, while NEWDAT was still set,
                  // ie. the previously stored message is lost.

                  CAN2_OBJ[0].MCR = 0xf7ff;  // reset MSGLST

                  // USER CODE BEGIN (CAN_IsrMsgLst,1)

                  // USER CODE END
                }
                else
                {
                  // The CAN controller has stored a new message
                  // into this object.

                  // USER CODE BEGIN (CAN_IsrRxOk,1)

		  CAN2_vGetMsgObj(1,&Msg);
		  P2_0 = !P2_0;
		  
                  // USER CODE END
                }

                CAN2_OBJ[0].MCR = 0xfdfd;  // reset NEWDAT, INTPND
              }

                   break;

     default:
              break;
    }  // end switch()
  }    // end while
}      // end ISR


//****************************************************************************
// @Function      void CAN2_vGetMsgObj(ubyte ObjNr, TCAN_Obj *pstObj)
//
//----------------------------------------------------------------------------
// @Description   This function fills the forwarded SW message object with the 
//                content of the chosen HW message object.
//                
//                The structure of the SW message object is defined in the 
//                header file CAN2.H (see TCAN_Obj).
//
//----------------------------------------------------------------------------
// @Returnvalue   none
//
//----------------------------------------------------------------------------
// @Parameters    Number of the message object to be read (1-15)
// @Parameters    Pointer on a message object to be filled by this function.
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:11 PM
//
//****************************************************************************

void CAN2_vGetMsgObj(ubyte ObjNr, TCAN_Obj *pstObj)
{
  ubyte i;
  ulong v;

  pstObj->ubMsgCfg = CAN2_OBJ[ObjNr-1].MCFG;

  v = 0;
  if(CAN2_OBJ[ObjNr-1].MCFG & 0x04)  // extended identifier
  {
    v += (CAN2_OBJ[ObjNr-1].LAR & 0xf800) >> 11;  // ID  4.. 0
    v += (CAN2_OBJ[ObjNr-1].LAR & 0x00ff) <<  5;  // ID 12.. 5
    v += (CAN2_OBJ[ObjNr-1].UAR & 0xff00) <<  5;  // ID 13..20
    v += (CAN2_OBJ[ObjNr-1].UAR & 0x00ff) << 21;  // ID 21..28
  }
  else                              // standard identifier 
  {
    v += (CAN2_OBJ[ObjNr-1].UAR & 0xe000) >> 13;  // ID 18..20
    v += (CAN2_OBJ[ObjNr-1].UAR & 0x00ff) <<  3;  // ID 21..28
  }
  pstObj->ulArbitr = v;

  for(i = 0; i < (CAN2_OBJ[ObjNr-1].MCFG & 0xf0) >> 4; i++)
  {
    pstObj->ubData[i] = CAN2_OBJ[ObjNr-1].Data[i];
  }

  pstObj->ubUser = CAN2_OBJ[ObjNr-1].Customer;

  if(ObjNr < 15 || (CAN2_OBJ[ObjNr-1].MCR & 0x000c) != 0x0008)
  {
    CAN2_OBJ[ObjNr-1].MCR = 0xfdff;  // reset NEWDAT
  }
}


// USER CODE BEGIN (CAN2_General,3)

// USER CODE END


