//****************************************************************************
// @Module        CAN2
// @Filename      CAN2.H
// @Project       can_hot
//----------------------------------------------------------------------------
// @Controller    Siemens C167CR-L
//
// @Compiler      Tasking C166 V5.0
//
// @Codegenerator DAvE 1-0-0
//
// @Description   This file contains all function prototypes and macros for CAN
//                
//
//----------------------------------------------------------------------------
// @Date          11/16/98 4:13:12 PM
//
//****************************************************************************

//****************************************************************************
// @Definition of a structure for the CAN2 data
//****************************************************************************

// The following data type serves as a software message object. Each access to
// a hardware message object has to be made by forward a pointer to a software
// message object (TCAN_Obj). The data type has the following fields:
//
// ubMsgCfg:
// this byte has the same structure as the message configuration register of a
// hardware message object. It contains the "Data Lenght Code" (DLC), the "Extended
// Identifier" (XTD) and the "Message Direction" (DIR, read only access).
//
//         7     6     5      4    3     2     1     0
//      |-----------------------------------------------|
//      |        DLC            | DIR | XTD |  0  |  0  |
//      |-----------------------------------------------|
//
// ulArbitr: 
// this field is four bytes long and contains either the 11-bit identifier 
// or the 29-bit identifier (as a HEX-Value)
//
// ubData[8]:
// 8 bytes containing the data of a frame
//
// ubUser:
// this field serves for storing application specific data. It will not be
// sended or received.
//


//****************************************************************************
// @Prototypes of global functions
//****************************************************************************

void CAN2_vInit(void);
void CAN2_vGetMsgObj(ubyte ObjNr, TCAN_Obj *pstObj);


// USER CODE BEGIN (CAN2Header,1)

// USER CODE END


//****************************************************************************
// @Macros
//****************************************************************************


// USER CODE BEGIN (CANHeader,2)

// USER CODE END


//****************************************************************************
// @Defines
//****************************************************************************

// Control/Status Register
#define C1CSR2   (*((uword volatile *) 0xEE00))

// Interrupt Register
#define C1IR2    (*((uword volatile *) 0xEE02))

// Bit Timing Register
#define C1BTR2   (*((uword volatile *) 0xEE04))

// Global Mask Short
#define C1GMS2   (*((uword volatile *) 0xEE06))

// Upper Global Mask Long
#define C1UGML2  (*((uword volatile *) 0xEE08))

// Lower Global Mask Long
#define C1LGML2  (*((uword volatile *) 0xEE0A))

// Upper Mask of Last Message
#define C1UMLM2  (*((uword volatile *) 0xEE0C))

// Lower Mask of Last Message
#define C1LMLM2  (*((uword volatile *) 0xEE0E))


// Structure for a single CAN object
// A total of 15 such object structures exists (starting at EF10H)

struct can2_obj 
{
  uword  MCR;       // Message Control Register
  uword  UAR;       // Upper Arbitration Register
  uword  LAR;       // Lower Arbitration Register
  ubyte  MCFG;      // Message Configuration Register
  ubyte  Data[8];   // Message Data 0 .. 7
  ubyte  Customer;  // Reserved for application specific data
};

#define CAN2_OBJ ((struct can2_obj volatile *) 0xEE10)


// USER CODE BEGIN (CAN2Header,3)

// USER CODE END


//****************************************************************************
// @Interrupt Vectors
//****************************************************************************

#define XP1INT   0x41

// USER CODE BEGIN (CAN2Header,4)

// USER CODE END

